@@ -10,11 +10,13 @@
 #include <string.h>
 #include <stdlib.h>
 #include <uim/uim.h>
+#include <uim/uim-helper.h>
 #include <uim/gettext.h>
 
 #define NR_CANDIDATES 10
 
-extern int uim_fd;
+static int im_uim_fd;
+static int read_tag;
 
 struct preedit_segment {
   int attr;
@@ -64,6 +66,8 @@ static void im_uim_class_init (GtkIMContextClass *class);
 
 static void show_preedit(GtkIMContext *ic, GtkWidget *preedit_label);
 
+static void helper_disconnect_cb(void);
+static void helper_read_cb(gpointer p, int fd, GdkInputCondition c);
 
 static const GTypeInfo class_info = {
   sizeof(IMContextUIMClass),
@@ -543,7 +547,7 @@ update_status_menu(IMUIMContext *uic)
   }
 
   /* menu */
-    uic->menu = gtk_option_menu_new();
+  uic->menu = gtk_option_menu_new();
   uic->stat_win = gtk_window_new(GTK_WINDOW_POPUP);
   menu = gtk_menu_new();
   for (i = 0; i < nr; i++) {
@@ -583,6 +587,18 @@ update_mode_list_cb(void *ptr)
   update_status_menu(uic);
 }
 
+static void
+update_prop_list_cb(void *ptr, char *str)
+{
+  printf("gtk prop list callbak %s\n",str);
+  uim_helper_send_message(im_uim_fd, str);
+}
+
+static void
+update_prop_label_cb(void *ptr, char *str)
+{
+  uim_helper_send_message(im_uim_fd, str);
+}
 
 static void
 select_candidate_cb(GtkWidget *clist,
@@ -682,8 +698,6 @@ layout_candidate(IMUIMContext *uic)
 
   trim_index(uic);
 
-  gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), 0, 1, &new_first_cand);
-
   sc_he = gdk_screen_get_height(gdk_screen_get_default ());
   sc_wi = gdk_screen_get_width (gdk_screen_get_default ());
 
@@ -705,25 +719,20 @@ layout_candidate(IMUIMContext *uic)
 
   gtk_window_move(GTK_WINDOW(uic->cwin.top_win), x, y );
 
-  new_first_cand = uim_get_candidate(uic->uc,uic->cwin.layout_begin);
-
+  new_first_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin);
+  gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), 0, 1, &old_first_cand);
   /* This condition works almost case, but not perfect.
      We must think out more good way.  */
 
   if( !old_first_cand ||
       strcmp( old_first_cand, new_first_cand) !=0 ) {
 
-    gtk_clist_freeze( GTK_CLIST(uic->cwin.clist));
+    gtk_clist_freeze(GTK_CLIST(uic->cwin.clist));
     gtk_clist_clear(GTK_CLIST(uic->cwin.clist));
 
-    buf[0] = "1";
-    buf[1] = new_first_cand;
-    gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
-    free( new_first_cand );
-
-    for (i = 1; i < NR_CANDIDATES; i++) {
+    for (i = 0; i < NR_CANDIDATES; i++) {
       if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
-	g_sprintf(idx, "%d", i + uic->cwin.layout_begin + 1 );
+	g_sprintf(idx, "%2d", i + uic->cwin.layout_begin + 1 );
 	buf[0] = idx;
 	buf[1] = uim_get_candidate(uic->uc, i + uic->cwin.layout_begin);
       } else {
@@ -739,6 +748,8 @@ layout_candidate(IMUIMContext *uic)
     gtk_clist_thaw( GTK_CLIST(uic->cwin.clist) );
   }
   
+  free( new_first_cand );
+
   uic->cwin.select_count ++;
   gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
 		       uic->cwin.candidate_index - uic->cwin.layout_begin,
@@ -787,6 +798,7 @@ im_module_create (const gchar *context_id)
 {
   GtkObject *obj = g_object_new(type_im_uim, NULL);
   IMUIMContext *uic = IM_UIM_CONTEXT(obj);
+
   uic->win = NULL;
   uic->menu = NULL;
   uic->stat_win = NULL;
@@ -801,19 +813,39 @@ im_module_create (const gchar *context_id)
   if (!strncmp(context_id, "uim-", 4)) {
     context_id = &context_id[4];
   }
+
+  im_uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);
+
+  if (im_uim_fd >= 0) {
+    read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
+			     helper_read_cb, 0);
+  }
+
+  uim_init();
+
+
   uic->uc =  uim_create_context(uic, "UTF-8",
 				NULL, (char *)context_id,
 				im_uim_commit_string);
+  /* to be a global context */
+  uim_helper_manage_context(uic->uc);
+  /**/
   uim_set_mode_cb(uic->uc, mode_cb);
   uim_set_preedit_cb(uic->uc, clear_cb,
 		     pushback_cb, update_cb);
   uim_set_mode_list_update_cb(uic->uc,
 			      update_mode_list_cb);
+  uim_set_prop_list_update_cb(uic->uc,
+			      update_prop_list_cb);
+  uim_set_prop_label_update_cb(uic->uc,
+			      update_prop_label_cb);
   uim_set_candidate_cb(uic->uc,
 		       cand_begin_cb,
 		       cand_update_cb,
 		       cand_end_cb);
   
+  uim_prop_list_update(uic->uc);
+
   uic->mode = uim_get_current_mode(uic->uc);
   uic->slave = g_object_new(GTK_TYPE_IM_CONTEXT_SIMPLE,
 			    NULL);
@@ -859,15 +891,47 @@ im_module_list(GtkIMContextInfo ***contexts,
   *n_contexts = nr;
 }
 
+static void
+helper_read_cb(gpointer p, int fd, GdkInputCondition c)
+{
+  /*  uim_helper_read_proc();*/
+  static char buf[1024];
+  char *result = NULL;
+  int i,rc;
+  while((i = uim_helpler_fd_readable(fd)) > 0) {
+
+    rc = read(fd, buf, sizeof(buf)-1);
+    
+    if(rc == 0) {
+      gdk_input_remove(read_tag);
+      return;
+    }
+
+    buf[rc] = '\0';
+    printf("read: %slength %d\n",buf, strlen(buf));
+    
+    
+    if(!result) {
+      result = (char*)malloc(strlen(buf)+1); 
+      result[0] = '\0';
+    } else {
+      result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
+    }
+   
+  strcat(result, buf);
+  }
+  free(result);
+}
+
+static void
+helper_disconnect_cb(void)
+{
+  gdk_input_remove(read_tag);
+}
+
 void
 im_module_init(GTypeModule *type_module)
 {
-  char *path = uim_helper_get_pathname();
-  uim_fd     = uim_helper_init_client_fd(path);
-  
-  free(path);
-
-  uim_init();
   context_list.next = (IMUIMContext *)&context_list;
   context_list.prev = (IMUIMContext *)&context_list;
   type_im_uim =
