@@ -196,6 +196,7 @@
       (im-pushback-mode-list id "Ҥ餬")
       (im-update-mode-list id)
       (im-update-mode id 0)
+      (anthy-update-prop-list id)
       ())))
 
 (define anthy-release-handler
@@ -684,18 +685,39 @@
       (set-car! (nthcdr seg (anthy-context-index-list ac)) idx)
       (anthy-update-preedit ac id))))
 
+(define anthy-prop-handler
+  (lambda (id message)
+    (if (= message
+	   "prop_anthy_hira")
+	())
+    (print "prop handler\n")
+	))
+
+(define anthy-update-prop-label
+  (lambda (ac)
+    (let (id (anthy-context-ac-id ac))
+    (im-update-prop-label "a\tϾ\nϥ⡼\n"))))
+  
+
 (define anthy-prop-list
   '(
     ("branch" "Ͼ")
-    ("" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "anthy_hira")
-    ("" "" "" "anthy_kana")
-    ("" "ѱѿ" "ѱѿ⡼" "anthy_zenkaku")
-    ("a" "ľ" "ե٥åȤϤǤޤ" "anthy_direct")    
+    ("leaf" "" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "prop_anthy_hira")
+    ("leaf" "" "" "" "prop_anthy_kana")
+    ("leaf" "" "ѱѿ" "ѱѿ⡼" "prop_anthy_zenkaku")
+    ("leaf" "a" "ľ" "ե٥åȤϤǤޤ" "prop_anthy_direct")    
+    ("branch" "ϥ⡼")
+    ("leaf" "" "޻" "޻" "prop_anthy_roma")
+    ("leaf" "" "" "" "prop_anthy_kana")
     ("branch" "ϥ⡼")
-    ("" "޻" "޻" "anthy_roma")
-    ("" "" "" "anthy_kana")
     ))
-    
+
+(define anthy-update-prop-list
+  (lambda (id)
+    (let (tmp)
+    (im-update-prop-list id "branch\tϾ\nleaf\t\tҤ餬\tdesc\tprop_anthy_hira\nbranch\t\tR\t޻\tdesc\tprop_anthy_roma\n")
+    )))
+
 (register-im
  'anthy
  "ja"
@@ -708,4 +730,6 @@
  anthy-release-key-handler
  anthy-reset-handler
  anthy-get-candidate-handler
- anthy-set-candidate-index-handler)
+ anthy-set-candidate-index-handler
+ anthy-prop-handler
+)
