@@ -806,4 +806,5 @@
  prime-release-key-handler
  prime-reset-handler
  prime-get-candidate-handler
- prime-set-candidate-index-handler)
+ prime-set-candidate-index-handler
+ nil)
