@@ -761,4 +761,5 @@
  skk-release-key-handler
  skk-reset-handler
  skk-get-candidate-handler
- skk-set-candidate-index-handler)
+ skk-set-candidate-index-handler
+ nil)
