@@ -1,5 +1,5 @@
-#include <sys/socket.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -72,8 +72,8 @@ int proc_func(int fd)
   FILE *rfile, *wfile;
 
 
-  rfile = fdopen(fd, "r");
-  wfile = fdopen(fd, "w");
+  /*  rfile = fdopen(fd, "r");
+      wfile = fdopen(fd, "w");*/
 
 
   /* use both fd and rfile is safe? I'm not sure...*/
@@ -81,10 +81,12 @@ int proc_func(int fd)
     //    fgets(buf, BUFFER_SIZE, rfile );
     printf("fd readable %d\n",fd);
     rc = read(fd,buf, sizeof(buf)-1);
-
+    
     if(rc == 0)
       return -1;
 
+    buf[rc] = '\0';
+    
     if(!content)
       content = (char*)malloc(strlen(buf)+1); 
     else
@@ -99,8 +101,6 @@ int proc_func(int fd)
   if( content ) {
     parse_content(content, fd);
   }
-    
-
   free(content);
 
   return 1;
@@ -113,38 +113,27 @@ static parse_content(char *content, int fd)
   if( strncmp(content,"new_context", 11) == 0 ) {
     
     printf("new context creating\n");
-    i = snprintf(buf,BUFFER_SIZE,"%d",
+    i = snprintf(buf,BUFFER_SIZE,"new_context:%d\n",
 		 uim_helper_server_new_context_id());
     printf("new context created %s\n",buf);
   }
-  /*  else if() {
-
-  }
-  else if() {
-
-  }
-  else if() {
-
-  }
-  else if() {
-
-  }*/
   else {
-
-    strcpy(buf,"OK\n");
-
+    for (i = 0; i < nclients; i ++){
+      if(clientfd[i] != fd && 
+	 uim_helpler_fd_writable(clientfd[i]))
+	write(clientfd[i],content,strlen(content));
+    }
+  strcpy(buf,"OK\n");
   }
-  
-  write(buf,strlen(buf)+1);
+  write(fd, buf, strlen(buf));
 
 }
 
 uim_helper_server_process_connection(int serv_fd, int (*proc_func)(int fd) )
 {
-  fd_set readfds;
-
   int i;
   int fd_biggest = 0;
+  fd_set readfds;
 
   fd_biggest = serv_fd;
 
@@ -180,8 +169,7 @@ uim_helper_server_process_connection(int serv_fd, int (*proc_func)(int fd) )
 
       file = fdopen(new_fd,"w");
 
-      fputs("OK", file);
-      fputc('\0', file);
+      fputs("OK\n", file);
 	
       clientfd[nclients] = new_fd;
       nclients++;
@@ -210,8 +198,9 @@ int
 main(int argc, char **argv)
 {
   char *path = uim_helper_get_pathname();
+  int serv_fd;
   unlink(path);
-  int serv_fd = init_serv_fd(path);
+  serv_fd = init_serv_fd(path);
 
   if (serv_fd < 0) {
     //    printf("Failed to init unix domain socket\n");
