@@ -1,6 +1,7 @@
 #include <unistd.h>
 #include <sys/socket.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -8,6 +9,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <uim/context.h>
 
 enum RorW
   {
@@ -19,31 +21,49 @@ char*
 uim_helper_send_message(int fd, char *message)
 {
   static char buf[1024];
-  char *result;
+  char *result = NULL;
   FILE *rfile, *wfile;
-  int i;
+  int i, rc;
 
   if(fd < 0)
     return "";
 
-  rfile = fdopen(fd, "r");
+  if(!message)
+    return "";
+  //  rfile = fdopen(fd, "r");
   wfile = fdopen(fd, "w");
   
   printf("send massage:%s\n",message);
 
   fputs(message, wfile);
-  fputc('\0',wfile);
 
   fflush(wfile);
+ 
+  /*  while((i = uim_helpler_fd_readable(fd)) > 0){
   
-  read(fd,buf, sizeof(buf)-1);
-    //  fgets(buf, sizeof(buf), rfile );
-  printf("replay: %s\n",buf);
-  
-  if(buf)
-    return buf;
+    rc = read(fd,buf, sizeof(buf)-1);
+
+    if(rc == 0)
+      return "";
+
+    buf[rc] = '\0';
+    
+    if(!result)
+      result = (char*)malloc(strlen(buf)+1); 
+    else
+      result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
+    
+      result[0] = '\0';
+    strcat(result, buf);
+  }
+
+    printf("replay: %s\n", result);
+    
+  if( result )
+    return result;
   else
     return "";
+  */
 }
 
 char *
@@ -59,15 +79,6 @@ uim_helper_get_pathname(void)
   return path;
 }
 
-int uim_helpler_fd_writable(int fd)
-{
-  return uim_helper_fd(fd, WRITE);
-}
-
-int uim_helpler_fd_readable(int fd)
-{
-  return uim_helper_fd(fd, READ);
-}
 
 static int
 uim_helper_fd(int fd, enum RorW rw)
@@ -93,4 +104,13 @@ uim_helper_fd(int fd, enum RorW rw)
     
     return FD_ISSET(fd,&fds) ? 1 : 0;
 }
- 
+
+int uim_helpler_fd_writable(int fd)
+{
+  return uim_helper_fd(fd, WRITE);
+}
+
+int uim_helpler_fd_readable(int fd)
+{
+  return uim_helper_fd(fd, READ);
+}
