@@ -26,7 +26,7 @@ int g_option_mask;
 int scr_width,scr_height;
 int host_byte_order;
 
-extern int uim_fd;
+static int lib_uim_fd;
 
 #define VERSION_NAME "uim-xim under the way! Version "PACKAGE_VERSION"\n"
 char *version_name=VERSION_NAME;
@@ -350,19 +350,32 @@ get_runtime_env()
     }
 }
 
+static void
+helper_read_cb(int fd, int ev)
+{
+  uim_helper_read_proc();
+}
+
+static void
+helper_disconnect_cb(void)
+{
+    remove_current_fd_watch(lib_uim_fd);
+}
+
 int
 main(int argc, char **argv)
 {
     int res;
     printf("UIM-XIM bridge. Now only supports Japanese.\n");
-    char *path = uim_helper_get_pathname();
 
     setlocale(LC_ALL, "");
     get_runtime_env();
     parse_args(argc, argv);
 
-    uim_fd = uim_helper_init_client_fd(path);
-    free(path);
+    lib_uim_fd = uim_helper_init_client_fd(helper_disconnect_cb);
+    if (lib_uim_fd >= 0) {
+	add_fd_watch(lib_uim_fd, READ_OK, helper_read_cb);
+    }
 
     res = uim_init();
     if (res) {
